@@ -68,9 +68,9 @@ class Brew(object):
         return self.__spawn("untap", [tap])
 
     def ls(self):
-        return self.__spawn(
+        return [formula for formula in self.__spawn(
             "ls", ["-1"], check_output=True
-        ).strip().split("\n")
+        ).strip().split("\n") if not formula.startswith("==>")]
 
     def uses(self, formula):
         args = ["--installed", "--recursive", formula]
@@ -141,7 +141,8 @@ def mkdir_p(path):
 def collapseuser(path):
     home_dir = os.environ.get("HOME", pwd.getpwuid(os.getuid()).pw_dir)
     if os.path.samefile(home_dir, commonpath([path, home_dir])):
-        return os.path.join("~", os.path.relpath(path, home_dir))
+        relpath = os.path.relpath(path, home_dir)
+        return os.path.join("~", relpath) if relpath != "." else "~"
     return path
 
 
