@@ -4,20 +4,22 @@ from . import _osx as osx
 from . import _tty as tty
 from .exceptions import (
     UnsupportedOSError, XcodeMissingError, BrewMissingError,
-    BootstrapMissingError, SymlinkError, AppMissingError
+    SymlinkError, AppMissingError, StowError
 )
 from ._sh import (
     Brew, Defaults, spawn, collapseuser, commonpath, curl, mkdir_p, read_json,
     write_json, modify_json, isdirname
 )
+from fnmatch import fnmatch
+from glob import iglob
 from rfc3987 import parse as urlparse
 from tempfile import mkdtemp
-from glob import iglob
 import errno
 import json
 import os
 import platform
 import re
+import shutil
 import subprocess
 import sys
 
@@ -26,16 +28,17 @@ _DEFAULTS_FALSE_RE = re.compile(r"\b(N(O)?|FALSE)\b", re.I)
 
 
 class Cider(object):
-    def __init__(self, cask=None, debug=None, verbose=None, cider_dir=None):
+    def __init__(self, cask=None, debug=None, verbose=None, cider_dir=None,
+                 support_dir=None):
         self.cask = cask if cask is not None else False
         self.debug = debug if debug is not None else False
         self.verbose = verbose if verbose is not None else False
         self.brew = Brew(cask, debug, verbose)
         self.defaults = Defaults(debug)
-        self.cider_dir = cider_dir if cider_dir is not None else os.path.join(
-            os.path.expanduser("~"),
-            ".cider"
-        )
+        self.cider_dir = cider_dir if cider_dir is not None else \
+            self.fallback_cider_dir()
+        self.support_dir = support_dir if support_dir is not None else \
+            self.fallback_support_dir()
 
     @property
     def symlink_dir(self):
@@ -49,31 +52,42 @@ class Cider(object):
     def defaults_file(self):
         return os.path.join(self.cider_dir, "defaults.json")
 
-    @property
-    def cache_dir(self):
-        return os.path.join(self.cider_dir, ".cache")
+    @staticmethod
+    def fallback_cider_dir():
+        try:
+            return os.path.join(os.environ["XDG_CONFIG_HOME"], "cider")
+        except KeyError:
+            return os.path.join(os.path.expanduser("~"), ".cider")
+
+    @staticmethod
+    def fallback_support_dir():
+        try:
+            return os.path.join(os.environ["XDG_DATA_HOME"], "cider")
+        except KeyError:
+            return os.path.join(
+                os.path.expanduser("~"),
+                "Library",
+                "Application Support",
+                "com.msanders.cider"
+            )
 
     @property
     def symlink_targets_file(self):
-        return os.path.join(self.cache_dir, "symlink_targets.json")
+        return os.path.join(self.support_dir, "symlink_targets.json")
 
     def read_bootstrap(self):
-        try:
-            return read_json(self.bootstrap_file)
-        except IOError as e:
-            if e.errno == errno.ENOENT:
-                raise BootstrapMissingError(
-                    "Bootstrap file not found. Expected at {0}".format(
-                        collapseuser(self.bootstrap_file)
-                    ),
-                    self.bootstrap_file
-                )
-
-            raise e
+        return read_json(self.bootstrap_file, {})
 
     def read_defaults(self):
         return read_json(self.defaults_file, {})
 
+    def _check_cider_dir(self):
+        if not os.path.isdir(self.cider_dir):
+            os.mkdir(self.cider_dir)
+            print(tty.progress("Created cider dir at {0}".format(
+                self.cider_dir
+            )))
+
     def _modify_bootstrap(self, key, transform=None, fallback=None):
         if transform is None:
             transform = lambda x: x
@@ -87,6 +101,7 @@ class Cider(object):
                 bootstrap[key] = sorted(bootstrap[key])
             return bootstrap
 
+        self._check_cider_dir()
         return modify_json(self.bootstrap_file, outer_transform)
 
     def _modify_defaults(self, domain, transform):
@@ -94,12 +109,14 @@ class Cider(object):
             defaults[domain] = transform(defaults.get(domain, {}))
             return defaults
 
+        self._check_cider_dir()
         return modify_json(self.defaults_file, outer_transform)
 
     def _cached_targets(self):
         return read_json(self.symlink_targets_file, [])
 
     def _update_target_cache(self, new_targets):
+        self._check_cider_dir()
         mkdir_p(os.path.dirname(self.symlink_targets_file))
         write_json(self.symlink_targets_file, sorted(new_targets))
 
@@ -117,6 +134,19 @@ class Cider(object):
                     collapseuser(target))
                 ))
 
+    @staticmethod
+    def _remove_link_target(source, target):
+        if os.path.exists(target):
+            if os.path.samefile(os.path.realpath(target),
+                                os.path.realpath(source)):
+                os.remove(target)
+            else:
+                raise SymlinkError(
+                    "{0} symlink target already exists at: {1}".format(
+                        collapseuser(source), collapseuser(target)
+                    )
+                )
+
     def _assert_requirements(self):
         macos_version = platform.mac_ver()[0]
 
@@ -138,6 +168,10 @@ class Cider(object):
                 "http://brew.sh/#install"
             )
 
+    @staticmethod
+    def _isstow(symlink, stow):
+        return symlink == stow or symlink.startswith(os.path.join(stow, ""))
+
     def restore(self):
         self._assert_requirements()
         caskbrew = Brew(True, self.debug, self.verbose)
@@ -252,6 +286,24 @@ class Cider(object):
             return os.path.join(expanded, os.path.basename(source))
         return expanded
 
+    def expandtargets(self, source_glob, target):
+        if not isdirname(target) and ("*" in source_glob or
+                                      "?" in source_glob):
+            raise SymlinkError(
+                "Invalid symlink: {0} => {1} (did you mean to add a "
+                "trailing '/'?)".format(source_glob, target)
+            )
+
+        mkdir_p(os.path.dirname(os.path.expanduser(target)))
+        sources = iglob(os.path.join(self.symlink_dir, source_glob))
+        expanded = []
+        for source in sources:
+            source = os.path.join(self.cider_dir, source)
+            source_target = self.expandtarget(source, target)
+
+            expanded.append((source, source_target))
+        return expanded
+
     def relink(self, force=None):
         force = force if force is not None else False
         symlinks = self.read_bootstrap().get("symlinks", {})
@@ -259,21 +311,10 @@ class Cider(object):
         new_targets = []
 
         for source_glob, target in symlinks.items():
-            if not isdirname(target) and ("*" in source_glob or
-                                          "?" in source_glob):
-                raise SymlinkError(
-                    "Invalid symlink: {0} => {1} (did you mean to add a "
-                    "trailing '/'?)".format(source_glob, target)
-                )
-
-            mkdir_p(os.path.dirname(os.path.expanduser(target)))
-            sources = iglob(os.path.join(self.symlink_dir, source_glob))
-            for source in sources:
-                source = os.path.join(self.cider_dir, source)
-                source_target = self.expandtarget(source, target)
-
-                if self.mklink(source, source_target, force):
-                    new_targets.append(source_target)
+            for source, expanded_target in self.expandtargets(source_glob,
+                                                              target):
+                if self.mklink(source, expanded_target, force):
+                    new_targets.append(expanded_target)
 
         self._remove_dead_targets(set(old_targets) - set(new_targets))
         self._update_target_cache(new_targets)
@@ -350,7 +391,7 @@ class Cider(object):
 
         def brew_orphan(formula):
             uses = self.brew.uses(formula)
-            return len(set(installed).intersection(set(uses))) == 0
+            return len(set(installed) & set(uses)) == 0
 
         return sorted(filter(brew_orphan, set(brewed) - set(installed)))
 
@@ -457,6 +498,95 @@ class Cider(object):
 
         tty.puts("Applied icons")
 
+    def add_symlink(self, name, target):
+        target = collapseuser(os.path.normpath(target))
+        target_dir = os.path.dirname(target)
+
+        # Add trailing slash for globbing.
+        if target_dir != "~":
+            target_dir = os.path.join(target_dir, "")
+
+        def transform(symlinks):
+            for key in symlinks:
+                if (os.path.dirname(key) == name and
+                   fnmatch(os.path.basename(target), os.path.basename(key))):
+                    return symlinks
+
+            dotted = os.path.basename(target).startswith(".")
+            pattern = "{0}/{1}*".format(name, "." if dotted else "")
+            symlinks[pattern] = target_dir
+            return symlinks
+
+        return self._modify_bootstrap("symlinks", transform, {})
+
+    def remove_symlink(self, name):
+        def transform(symlinks):
+            if symlinks:
+                for key in symlinks.keys():
+                    if self._isstow(key, name):
+                        del symlinks[key]
+
+            return symlinks
+
+        return self._modify_bootstrap("symlinks", transform)
+
+    def addlink(self, name, path):
+        stow_path = os.path.join(self.symlink_dir, name)
+        stow_fpath = os.path.join(stow_path, os.path.basename(path))
+        if not os.path.exists(path):
+            raise StowError("Can't link {0}: No such file or directory".format(
+                collapseuser(path)
+            ))
+
+        samefile = os.path.exists(stow_fpath) and os.path.samefile(
+            os.path.realpath(stow_fpath), os.path.realpath(path)
+        )
+
+        if os.path.exists(stow_fpath) and not samefile:
+            raise StowError("Link already exists at {0}".format(
+                collapseuser(stow_fpath)
+            ))
+
+        if not samefile:
+            mkdir_p(stow_path)
+            shutil.move(path, stow_path)
+
+        target = os.path.abspath(path)
+        self.add_symlink(name, target)
+        self.mklink(stow_fpath, target)
+        self._update_target_cache(self._cached_targets() + [target])
+
+    def unlink(self, name):
+        symlinks = self.read_bootstrap().get("symlinks", {})
+
+        removed_targets = set()
+        found = False
+        for source_glob, target in symlinks.items():
+            if self._isstow(source_glob, name):
+                found = True
+                for source, target in self.expandtargets(source_glob, target):
+                    self._remove_link_target(source, target)
+                    removed_targets.add(target)
+                    shutil.move(source, target)
+                    print(tty.progress("Moved {0} -> {1}".format(
+                        collapseuser(source),
+                        collapseuser(target)
+                    )))
+
+        if not found:
+            raise StowError("No symlink found with name: {0}".format(name))
+
+        try:
+            os.rmdir(os.path.join(self.symlink_dir, name))
+        except OSError as e:
+            if e.errno != errno.ENOTEMPTY:
+                raise e
+
+        self.remove_symlink(name)
+        self._update_target_cache(
+            set(self._cached_targets()) - removed_targets
+        )
+
 
 def _apply_icon(app, icon):
     app_path = osx.path_for_app(app)
