@@ -111,22 +111,30 @@ class TestCiderCLI(object):
             ("remove-icon", "remove_icon"), [app], debug=debug, verbose=verbose
         )
 
-    def apply_icons(self, debug, verbose):
+    def test_apply_icons(self, debug, verbose):
         _test_command(
             ("apply-icons", "apply_icons"), debug=debug, verbose=verbose
         )
 
-    def run_scripts(self, debug, verbose):
+    def test_run_scripts(self, debug, verbose):
         _test_command(
             ("run-scripts", "run_scripts"), debug=debug, verbose=verbose
         )
 
-    def restore(self, debug, verbose):
+    def test_restore(self, debug, verbose):
         _test_command("restore", debug=debug, verbose=verbose)
 
-    def relink(self, debug, verbose):
+    def test_relink(self, debug, verbose):
         _test_command("relink", debug=debug, verbose=verbose)
 
+    @pytest.mark.randomize(source=str, name=str, min_length=1)
+    def test_addlink(self, debug, verbose, source, name):
+        _test_command("addlink", [name, source], debug=debug, verbose=verbose)
+
+    @pytest.mark.randomize(name=str, min_length=1)
+    def test_unlink(self, debug, verbose, name):
+        _test_command("unlink", [name], debug=debug, verbose=verbose)
+
     @staticmethod
     def __global_domain_params(name, key):
         return "NSGlobalDomain", name, key
